@@ -13,9 +13,15 @@ use warnings;
 use strict;
 
 package Text::VimColor;
+{
+  $Text::VimColor::VERSION = '0.20';
+}
+# git description: v0.19-0-ga57a8f6
+
+BEGIN {
+  $Text::VimColor::AUTHORITY = 'cpan:RWSTAUNER';
+}
 # ABSTRACT: Syntax highlight text using Vim
-our $VERSION = '0.13'; # VERSION
-our $AUTHORITY = 'cpan:RWSTAUNER'; # AUTHORITY
 
 use IO::File;
 use File::Copy qw( copy );
@@ -23,6 +29,8 @@ use File::ShareDir ();
 use File::Temp qw( tempfile );
 use Path::Class qw( file );
 use Carp;
+use IPC::Open3 (); # core
+use Symbol (); # core
 
 # for backward compatibility
 our $SHARED = File::ShareDir::dist_dir('Text-VimColor');
@@ -63,7 +71,7 @@ our %ANSI_COLORS = (
 );
 
 # Set to true to print the command line used to run Vim.
-our $DEBUG = 0;
+our $DEBUG = $ENV{TEXT_VIMCOLOR_DEBUG};
 
 sub new
 {
@@ -142,18 +150,27 @@ sub ansi
    my $syntax = $self->marked;
 
    require Term::ANSIColor;
-   # allow the environment to overwrite;
-   my %colors = (%ANSI_COLORS,
-      split(/\s*[=;]\s*/, $ENV{TEXT_VIMCOLOR_ANSI} || '')
-   );
+  # allow the environment to overwrite:
+  my %colors = (
+    %ANSI_COLORS,
+    $ENV{TEXT_VIMCOLOR_ANSI} ? split(/\s*[=;]\s*/, $ENV{TEXT_VIMCOLOR_ANSI}) : ()
+  );
 
-   my $ansi = '';
-   foreach (@$syntax) {
-      $ansi .= $_->[1], next
-         if $_->[0] eq '';
+  local $_;
 
-      $ansi .= Term::ANSIColor::colored([ $colors{ $_->[0] } ], $_->[1]);
-   }
+  # Term::ANSIColor didn't support bright values until version 3
+  # Handle this here to cover custom colors and not require T::AC until needed
+  if( Term::ANSIColor->VERSION < 3 ){
+    s/bright_// for values %colors;
+  }
+
+  # compared to join/map or foreach/my this benched as the fastest:
+  my $ansi = '';
+  for ( @$syntax ){
+    $ansi .= $_->[0] eq ''
+      ? $_->[1]
+      : Term::ANSIColor::colored([ $colors{ $_->[0] } ], $_->[1]);
+  }
 
    return $ansi;
 }
@@ -303,6 +320,10 @@ sub _do_markup
    croak "Text::VimColor syntax script '$vim_syntax_script' not installed"
       unless -f $vim_syntax_script && -r $vim_syntax_script;
 
+   if ($DEBUG) {
+      print STDERR __PACKAGE__."::_do_markup: script: $vim_syntax_script\n";
+   }
+
    my $filename = $self->{file};
    my $input_is_temporary = 0;
    if (ref $self->{file}) {
@@ -339,21 +360,48 @@ sub _do_markup
    my $filetype = $self->{filetype};
    my $filetype_set = defined $filetype ? ":set filetype=$filetype" : '';
    my $vim_let = $self->{vim_let};
-   print $script_fh (map { ":let $_=$vim_let->{$_}\n" }
-                     grep { defined $vim_let->{$_} }
-                     keys %$vim_let),
-                    ":filetype on\n",
-                    "$filetype_set\n",
-                    ":source $vim_syntax_script\n",
-                    ":write! $out_filename\n",
-                    ":qall!\n";
+
+  # on linux '-s' is fast and '--cmd' adds the 2-second startup delay
+  # are there situations where --cmd is necessary or useful?
+  # XXX: for debugging, may be removed in the future
+  my $use_cmd_opt = $ENV{TEXT_VIMCOLOR_CMD_OPT};
+
+  # Specify filename as argument to command (rather than using :edit in script).
+  # If using --cmd then the filename needs to be in the script.
+  # For some reason windows doesn't seem to like the filename being in the arg list.
+  # Are there other times that this is needed?
+  my $file_as_arg = ($use_cmd_opt || $^O ne 'MSWin32');
+
+  my @script_lines = (
+    map { "$_\n" }
+      # do :edit before :let or the buffer variables may get reset
+      (!$file_as_arg ? ":edit $filename" : ()),
+      (
+        map  { ":let $_=$vim_let->{$_}" }
+        grep { defined  $vim_let->{$_} }
+          keys %$vim_let
+      ),
+      ':filetype on',
+       $filetype_set,
+      ":source $vim_syntax_script",
+      ":write! $out_filename",
+      ':qall!',
+  );
+
+  print STDERR map { __PACKAGE__ . " | $_" } @script_lines if $DEBUG;
+
+   print $script_fh @script_lines;
    close $script_fh;
 
    $self->_run(
       $self->{vim_command},
       @{$self->{vim_options}},
-      $filename,
-      '-s', $script_filename,
+      ($file_as_arg ? $filename : ()),
+      (
+        $use_cmd_opt
+          ? ( '--cmd' => "silent! so $script_filename" )
+          : ( '-s'    => $script_filename )
+      ),
    );
 
    unlink $filename
@@ -384,6 +432,9 @@ sub _add_markup
 {
    my ($syntax, $type, $text) = @_;
 
+   # TODO: make this optional
+   # (https://github.com/petdance/vim-perl/blob/master/t/01_highlighting.t#L12)
+
    # Ignore types we don't know about.  At least one syntax file (xml.vim)
    # can produce these.  It happens when a syntax type isn't 'linked' to
    # one of the predefined types.
@@ -412,64 +463,48 @@ sub _run
    my ($self, $prog, @args) = @_;
 
    if ($DEBUG) {
-      ## no critic (MutatingListFunctions)
       print STDERR __PACKAGE__."::_run: $prog " .
-            join(' ', map { s/'/'\\''/g; "'$_'" } @args) . "\n";
+            join(' ', map { "'$_'" } @args) . "\n";
    }
 
-   my ($err_fh, $err_filename) = tempfile();
-   my $old_fh = select($err_fh);
-   $| = 1;
-   select($old_fh);
+  {
+    my ($in, $out) = (Symbol::gensym(), Symbol::gensym());
+    my $err_fh = Symbol::gensym();
+
+    my $pid = IPC::Open3::open3($in, $out, $err_fh, $prog => @args);
+
+    # close these to avoid any ambiguity that might cause this to block
+    # (see also the paragraph about "select" in IPC::Open3)
+    close($in);
+    close($out);
+
+    # read handle before waitpid to avoid hanging on older systems
+    my $errout = do { local $/; <$err_fh> };
 
-   my $pid = fork;
-   if ($pid) {
       my $gotpid = waitpid($pid, 0);
       croak "couldn't run the program '$prog'" if $gotpid == -1;
       my $error = $? >> 8;
       if ($error) {
-         seek $err_fh, 0, 0;
-         my $errout = do { local $/; <$err_fh> };
          $errout =~ s/\n+\z//;
-         close $err_fh;
-         unlink $err_filename;
          my $details = $errout eq '' ? '' :
                        "\nVim wrote this error output:\n$errout\n";
          croak "$prog returned an error code of '$error'$details";
       }
-      close $err_fh;
-      unlink $err_filename;
-   }
-   else {
-      defined $pid
-         or croak "error forking to run $prog: $!";
-      tied $_ and untie $_ for *STDOUT, *STDERR, *STDIN;
-      ## no critic (TwoArgOpen)
-      open STDIN, '/dev/null';
-      open STDOUT, '>/dev/null';
-      open STDERR, '>&=' . fileno($err_fh)
-         or croak "can't connect STDERR to temporary file '$err_filename': $!";
-      exec $prog $prog, @args;
-      die "\n";   # exec() will already have sent a suitable error message.
    }
 }
 
 1;
 
 
-# Local Variables:
-# mode: perl
-# perl-indent-level: 3
-# perl-continued-statement-offset: 3
-# End:
-
 __END__
 =pod
 
+=encoding utf-8
+
 =for :stopwords Geoff Richards Randy Stauner ACKNOWLEDGEMENTS ansi html xml DOCTYPE XHTML
-XSL XSLT XSL-FO pdf inline stylesheet filetype PreProc Todo Moolenaar cpan
-testmatrix url annocpan anno bugtracker rt cpants kwalitee diff irc mailto
-metadata placeholders
+XSL XSLT XSL-FO pdf inline stylesheet filetype PreProc Todo TODO syntaxes
+Moolenaar cpan testmatrix url annocpan anno bugtracker rt cpants kwalitee
+diff irc mailto metadata placeholders metacpan
 
 =head1 NAME
 
@@ -477,7 +512,7 @@ Text::VimColor - Syntax highlight text using Vim
 
 =head1 VERSION
 
-version 0.13
+version 0.20
 
 =head1 SYNOPSIS
 
@@ -922,8 +957,40 @@ Vim is run.
 
 =item *
 
-It doesn't work on Windows.  I am unlikely to fix this, but if anyone
-who knows Windows can sort it out let me know.
+This requires vim version 6 (it has since 2003).
+There may be workarounds to support version 5 (technically 5.4+).
+Upgrading vim is a much better idea, but if you need support
+for older versions please file a ticket (with patches if possible).
+
+=back
+
+=head1 TODO
+
+=over 4
+
+=item *
+
+L<https://github.com/rwstauner/Text-VimColor/issues/1>
+
+=item *
+
+option for 'set number'
+
+=item *
+
+extra_vim_options (additional instead of overwriting defaults)
+
+=item *
+
+make global vars available through methods
+
+=item *
+
+test constructor and then simplify it: copy default values into it
+
+=item *
+
+list available syntaxes? (see L<IkiWiki::Plugin::syntax::Vim>)
 
 =back
 
